'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _transforms = require('../transforms');

var _transforms2 = _interopRequireDefault(_transforms);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Change.
 *
 * @type {Change}
 */

var Change = function () {

  /**
   * Constructor.
   *
   * @param {State} state
   * @param {Array} operations (optional)
   */

  function Change(state, operations) {
    _classCallCheck(this, Change);

    this.state = state;
    this.operations = [];
    this.save = null;

    if (operations) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = operations[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var op = _step.value;

          this.operate(op);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }

  /**
   * Get the kind.
   *
   * @return {String}
   */

  _createClass(Change, [{
    key: 'save',


    /**
     * Get or set whether the change should be saved.
     *
     * @param {Boolean} save
     * @return {Transform}
     */

    value: function save(_save) {
      if (_save == null) return this._save;
      this._save = !!_save;
      return this;
    }

    /**
     * Mark the change as being native.
     *
     * @param {Boolean} isNative (optional)
     * @return {Transform}
     */

  }, {
    key: 'isNative',
    value: function isNative(_isNative) {
      if (_isNative == null) _isNative = true;
      this.isNative = !!_isNative;
      return this;
    }

    /**
     * Apply the change to the editor.
     *
     * @param {Object} options
     */

  }, {
    key: 'apply',
    value: function apply() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var history = this.history;
      var state = this.state;
      var operations = this.operations;
      var _state = state;
      var selection = _state.selection;

      // If there are cursor marks and no mark operations have been performed,
      // then remove the cursor marks.

      if (selection.marks) {
        var hasMark = operations.some(function (op) {
          return op.type == 'add-mark' || op.type == 'remove-mark';
        });

        if (!hasMark) {
          selection = selection.merge({ marks: null });
          state = state.merge({ selection: selection });
        }
      }

      // Determine whether we need to create a new snapshot.
      var snapshot = options.isDistinct == null ? this.shouldSnapshot() : options.isDistinct;

      // If we should, save a snapshot into the history before transforming.
      if (snapshot) history.save(state);

      // Apply the "isNative" flag, which is used to allow for natively-handled
      // content changes to skip rerendering the editor for performance.
      state = state.merge({
        isNative: !!options.isNative
      });

      return state;
    }

    /**
     * Check if a change has an operation by `type`.
     *
     * @param {String} type
     * @return {Boolean}
     */

  }, {
    key: 'has',
    value: function has(type) {
      return this.operations.some(function (operation) {
        return operation.type == type;
      });
    }

    /**
     * Check if a change has only operations of `type`.
     *
     * @param {String} type
     * @return {Boolean}
     */

  }, {
    key: 'hasOnly',
    value: function hasOnly(type) {
      return this.operations.every(function (operation) {
        return operation.type == type;
      });
    }
  }, {
    key: 'kind',
    get: function get() {
      return 'change';
    }
  }]);

  return Change;
}();

/**
 * Export.
 *
 * @type {Change}
 */

exports.default = Change;