'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _transforms = require('../transforms');

var _transforms2 = _interopRequireDefault(_transforms);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Transform.
 *
 * @type {Transform}
 */

var Transform = function () {

  /**
   * Constructor.
   *
   * @param {Object} properties
   */

  function Transform(properties) {
    _classCallCheck(this, Transform);

    var state = properties.state;

    this.state = state;
    this.operations = [];
  }

  /**
   * Get the kind.
   *
   * @return {String} kind
   */

  _createClass(Transform, [{
    key: 'apply',


    /**
     * Apply the transform and return the new state.
     *
     * @param {Object} options
     *   @property {Boolean} isNative
     *   @property {Boolean} merge
     *   @property {Boolean} save
     * @return {State} state
     */

    value: function apply() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var merge = options.merge;
      var _options$isNative = options.isNative;
      var isNative = _options$isNative === undefined ? false : _options$isNative;
      var _options$save = options.save;
      var save = _options$save === undefined ? true : _options$save;
      var state = this.state;
      var operations = this.operations;
      var history = state.history;
      var undos = history.undos;
      var redos = history.redos;

      var previous = undos.peek();

      // If there are no operations, abort early.
      if (!operations.length) return state;

      // If there's a previous save point, determine if the new operations should
      // be merged into the previous ones.
      if (previous && merge == null) {
        var types = operations.map(function (op) {
          return op.type;
        });
        var prevTypes = previous.map(function (op) {
          return op.type;
        });
        var edits = types.filter(function (type) {
          return type != 'set_selection';
        });
        var prevEdits = prevTypes.filter(function (type) {
          return type != 'set_selection';
        });
        var onlySelections = types.every(function (type) {
          return type == 'set_selection';
        });
        var onlyInserts = edits.length && edits.every(function (type) {
          return type == 'insert_text';
        });
        var onlyRemoves = edits.length && edits.every(function (type) {
          return type == 'remove_text';
        });
        var prevOnlyInserts = prevEdits.length && prevEdits.every(function (type) {
          return type == 'insert_text';
        });
        var prevOnlyRemoves = prevEdits.length && prevEdits.every(function (type) {
          return type == 'remove_text';
        });

        merge = onlySelections || onlyInserts && prevOnlyInserts || onlyRemoves && prevOnlyRemoves;
      }

      // Save the new operations.
      if (save || !previous) {
        this.save({ merge: merge });
      }

      // Return the new state with the `isNative` flag set.
      return this.state.merge({ isNative: !!isNative });
    }
  }, {
    key: 'kind',
    get: function get() {
      return 'transform';
    }
  }]);

  return Transform;
}();

/**
 * Add a transform method for each of the transforms.
 */

Object.keys(_transforms2.default).forEach(function (type) {
  Transform.prototype[type] = function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _transforms2.default[type].apply(_transforms2.default, [this].concat(args));
  };
});

/**
 * Export.
 *
 * @type {Transform}
 */

exports.default = Transform;