'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _normalizeMark = require('../../utils/normalize-mark');

var _normalizeMark2 = _interopRequireDefault(_normalizeMark);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Add a `mark` to the characters in the current selection.
 *
 * @param {State} state
 * @param {Mark} mark
 * @return {State} state
 */

function addMark(state, mark) {
  mark = (0, _normalizeMark2.default)(mark);
  var _state = state;
  var cursorMarks = _state.cursorMarks;
  var document = _state.document;
  var selection = _state.selection;

  // If the selection is collapsed, add the mark to the cursor instead.

  if (selection.isCollapsed) {
    var marks = document.getMarksAtRange(selection);
    state = state.merge({ cursorMarks: marks.add(mark) });
    return state;
  }

  document = document.addMarkAtRange(selection, mark);
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = addMark;