'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _splitBlockAtRange = require('./split-block-at-range');

var _splitBlockAtRange2 = _interopRequireDefault(_splitBlockAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Delete everything in a `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @return {State} state
 */

function deleteAtRange(state, range) {
  var _state = state;
  var document = _state.document;

  // If the range is collapsed, there's nothing to delete.

  if (range.isCollapsed) return state;

  // Make sure the children exist.
  var startKey = range.startKey;
  var startOffset = range.startOffset;
  var endKey = range.endKey;
  var endOffset = range.endOffset;

  document.assertDescendant(startKey);
  document.assertDescendant(endKey);

  // If the start and end nodes are the same, just remove characters.
  if (startKey == endKey) {
    var text = document.getDescendant(startKey);
    text = text.removeCharacters(startOffset, endOffset);
    document = document.updateDescendant(text);
    return document.normalize();
  }

  // Split the blocks and determine the edge boundaries.
  var start = range.collapseToStart();
  var end = range.collapseToEnd();
  var ancestor = document.getCommonAncestor(startKey, endKey);
  var isAncestor = ancestor == document;

  state = (0, _splitBlockAtRange2.default)(state, start, Infinity);
  state = (0, _splitBlockAtRange2.default)(state, end, Infinity);
  document = state.document;

  var startText = ancestor.getDescendant(startKey);
  var startEdgeText = ancestor.getNextText(startKey);

  var endText = ancestor.getNextText(endKey);
  var endEdgeText = ancestor.getDescendant(endKey);

  // Remove the new blocks inside the edges.
  var startEdgeBlock = ancestor.getFurthestBlock(startEdgeText);
  var endEdgeBlock = ancestor.getFurthestBlock(endEdgeText);

  var nodes = ancestor.nodes.takeUntil(function (n) {
    return n == startEdgeBlock;
  }).concat(ancestor.nodes.skipUntil(function (n) {
    return n == endEdgeBlock;
  }).rest());

  ancestor = ancestor.merge({ nodes: nodes });

  // Take the end edge's inline nodes and move them to the start edge.
  var startBlock = ancestor.getClosestBlock(startText);
  var endBlock = ancestor.getClosestBlock(endText);

  var startNodes = startBlock.nodes.concat(endBlock.nodes);
  startBlock = startBlock.merge({ nodes: startNodes });
  ancestor = ancestor.updateDescendant(startBlock);

  // While the end child is an only child, remove the block it's in.
  var endParent = ancestor.getClosestBlock(endBlock);

  while (endParent && endParent.nodes.size == 1) {
    endBlock = endParent;
    endParent = ancestor.getClosestBlock(endParent);
  }

  ancestor = ancestor.removeDescendant(endBlock);

  // Update the document.
  document = isAncestor ? ancestor : document.updateDescendant(ancestor);

  // Normalize the adjacent text nodes.
  document = document.normalize();

  // Update the state.
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = deleteAtRange;