'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Delete forward `n` characters at a `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Number} n (optional)
 * @return {State} state
 */

function deleteForwardAtRange(state, range) {
  var n = arguments.length <= 2 || arguments[2] === undefined ? 1 : arguments[2];
  var _state = state;
  var document = _state.document;
  var _range = range;
  var startKey = _range.startKey;

  // When the range is still expanded, just do a regular delete.

  if (range.isExpanded) return (0, _deleteAtRange2.default)(state, range);

  // When collapsed at the end of the node, there's nothing to do.
  if (range.isAtEndOf(document)) return state;

  // When collapsed in a void node, remove that node.
  var block = document.getClosestBlock(startKey);
  if (block && block.isVoid) {
    document = document.removeDescendant(block);
    state = state.merge({ document: document });
    return state;
  }

  var inline = document.getClosestInline(startKey);
  if (inline && inline.isVoid) {
    document = document.removeDescendant(inline);
    state = state.merge({ document: document });
    return state;
  }

  // When at end of a text node, merge forwards into the next text node.
  var startNode = document.getDescendant(startKey);
  if (range.isAtEndOf(startNode)) {
    var next = document.getNextText(startNode);
    range = range.extendToStartOf(next);
    range = range.normalize(document);
    return (0, _deleteAtRange2.default)(state, range);
  }

  // Otherwise, remove `n` characters ahead of the cursor.
  range = range.extendForward(n);
  range = range.normalize(document);
  return (0, _deleteAtRange2.default)(state, range);
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = deleteForwardAtRange;