'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _block = require('../../models/block');

var _block2 = _interopRequireDefault(_block);

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

var _normalizeBlock = require('../../utils/normalize-block');

var _normalizeBlock2 = _interopRequireDefault(_normalizeBlock);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Insert a `block` node at `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Block or String or Object} block
 * @return {State} state
 */

function insertBlockAtRange(state, range, block) {
  var _state = state;
  var document = _state.document;

  // Normalize the block argument.

  block = (0, _normalizeBlock2.default)(block);

  // If expanded, delete the range first.
  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  var _range = range;
  var startKey = _range.startKey;
  var startOffset = _range.startOffset;

  var startBlock = document.getClosestBlock(startKey);
  var parent = document.getParent(startBlock);
  var nodes = _block2.default.createList([block]);
  var isParent = parent == document;

  // If the start block is void, insert after it.
  if (startBlock.isVoid) {
    parent = parent.insertChildrenAfter(startBlock, nodes);
  }

  // If the block is empty, replace it.
  else if (startBlock.isEmpty) {
      parent = parent.insertChildrenAfter(startBlock, nodes);
      parent = parent.removeDescendant(startBlock);
    }

    // If the range is at the start of the block, insert before.
    else if (range.isAtStartOf(startBlock)) {
        parent = parent.insertChildrenBefore(startBlock, nodes);
      }

      // If the range is at the end of the block, insert after.
      else if (range.isAtEndOf(startBlock)) {
          parent = parent.insertChildrenAfter(startBlock, nodes);
        }

        // Otherwise, split the block and insert between.
        else {
            document = document.splitBlockAtRange(range);
            parent = document.getParent(startBlock);
            startBlock = document.getClosestBlock(startKey);
            nodes = parent.nodes.takeUntil(function (n) {
              return n == startBlock;
            }).push(startBlock).push(block).concat(parent.nodes.skipUntil(function (n) {
              return n == startBlock;
            }).rest());
            parent = parent.merge({ nodes: nodes });
          }

  // Update the document.
  document = isParent ? parent : document.updateDescendant(parent);

  // Normalize the document.
  document = document.normalize();

  // Return the updated state.
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = insertBlockAtRange;