'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

var _normalizeInline = require('../../utils/normalize-inline');

var _normalizeInline2 = _interopRequireDefault(_normalizeInline);

var _splitTextAtRange = require('./split-text-at-range');

var _splitTextAtRange2 = _interopRequireDefault(_splitTextAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Insert an `inline` node at `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Inline or String or Object} inline
 * @return {State} state
 */

function insertInlineAtRange(state, range, inline) {
  var _state = state;
  var document = _state.document;

  // Normalize the inline argument.

  inline = (0, _normalizeInline2.default)(inline);

  // If expanded, delete the range first.
  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  var _range = range;
  var startKey = _range.startKey;
  var endKey = _range.endKey;
  var startOffset = _range.startOffset;
  var endOffset = _range.endOffset;

  // If the range is inside a void, abort.

  var startBlock = document.getClosestBlock(startKey);
  if (startBlock && startBlock.isVoid) return state;

  var startInline = document.getClosestInline(startKey);
  if (startInline && startInline.isVoid) return state;

  // Split the text nodes at the cursor.
  state = (0, _splitTextAtRange2.default)(state, range);
  document = state.document;

  // Insert the inline between the split text nodes.
  var startText = document.getDescendant(startKey);
  var parent = document.getParent(startKey);
  var nodes = parent.nodes.takeUntil(function (n) {
    return n == startText;
  }).push(startText).push(inline).concat(parent.nodes.skipUntil(function (n) {
    return n == startText;
  }).rest());

  parent = parent.merge({ nodes: nodes });
  document = document.updateDescendant(parent);
  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = insertInlineAtRange;