'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Insert text `string` at a `range`, with optional `marks`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {String} string
 * @param {Set} marks (optional)
 * @return {State} state
 */

function insertTextAtRange(state, range, string, marks) {
  var _state = state;
  var document = _state.document;
  var _range = range;
  var startKey = _range.startKey;
  var startOffset = _range.startOffset;

  var isVoid = document.hasVoidParent(startKey);

  // If inside a void node, do nothing.
  if (isVoid) return state;

  // Is the range is expanded, delete it first.
  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  // Insert text at the range's offset.
  var text = document.getDescendant(startKey);
  text = text.insertText(startOffset, string, marks);
  document = document.updateDescendant(text);

  // Return the updated selection.
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = insertTextAtRange;