'use strict';

var _isInRange = require('../../utils/is-in-range');

var _isInRange2 = _interopRequireDefault(_isInRange);

var _normalizeMark = require('../../utils/normalize-mark');

var _normalizeMark2 = _interopRequireDefault(_normalizeMark);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Remove an existing `mark` to the characters at `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Mark or String} mark (optional)
 * @return {State} state
 */

function removeMarkAtRange(state, range, mark) {
  mark = (0, _normalizeMark2.default)(mark);
  var _state = state;
  var document = _state.document;

  // When the range is collapsed, do nothing.

  if (range.isCollapsed) return state;

  // Otherwise, find each of the text nodes within the range.
  var texts = document.getTextsAtRange(range);

  // Apply the mark to each of the text nodes's matching characters.
  texts = texts.map(function (text) {
    var characters = text.characters.map(function (char, i) {
      if (!(0, _isInRange2.default)(i, text, range)) return char;
      var marks = char.marks;

      marks = mark ? marks.remove(mark) : marks.clear();
      return char.merge({ marks: marks });
    });

    return text.merge({ characters: characters });
  });

  // Update each of the text nodes.
  texts.forEach(function (text) {
    document = document.updateDescendant(text);
  });

  state = state.merge({ document: document });
  return state;
}