'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _block = require('../../models/block');

var _block2 = _interopRequireDefault(_block);

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

var _splitInlineAtRange = require('./split-inline-at-range');

var _splitInlineAtRange2 = _interopRequireDefault(_splitInlineAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Split the block nodes at a `range`, to optional `depth`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Number} depth (optional)
 * @return {State} state
 */

function splitBlockAtRange(state, range) {
  var depth = arguments.length <= 2 || arguments[2] === undefined ? 1 : arguments[2];
  var _state = state;
  var document = _state.document;

  // If the range is expanded, remove it first.

  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  // Split the inline nodes at the range.
  state = (0, _splitInlineAtRange2.default)(state, range);
  document = state.document;

  // Find the highest inline elements that were split.
  var _range = range;
  var startKey = _range.startKey;

  var firstText = document.getDescendant(startKey);
  var secondText = document.getNextText(startKey);
  var firstChild = document.getFurthestInline(firstText) || firstText;
  var secondChild = document.getFurthestInline(secondText) || secondText;
  var parent = document.getClosestBlock(firstChild);
  var firstChildren = void 0;
  var secondChildren = void 0;
  var d = 0;

  // While the parent is a block, split the block nodes.
  while (parent && d < depth) {
    firstChildren = parent.nodes.takeUntil(function (n) {
      return n == firstChild;
    }).push(firstChild);
    secondChildren = parent.nodes.skipUntil(function (n) {
      return n == secondChild;
    });
    firstChild = parent.merge({ nodes: firstChildren });
    secondChild = _block2.default.create({
      nodes: secondChildren,
      type: parent.type,
      data: parent.data
    });

    // Add the new children.
    var grandparent = document.getParent(parent);
    var nodes = grandparent.nodes.takeUntil(function (n) {
      return n.key == firstChild.key;
    }).push(firstChild).push(secondChild).concat(grandparent.nodes.skipUntil(function (n) {
      return n.key == firstChild.key;
    }).rest());

    // Update the grandparent.
    document = grandparent == document ? document.merge({ nodes: nodes }) : document.updateDescendant(grandparent.merge({ nodes: nodes }));

    d++;
    parent = document.getClosestBlock(firstChild);
  }

  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = splitBlockAtRange;