'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _inline = require('../../models/inline');

var _inline2 = _interopRequireDefault(_inline);

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

var _splitTextAtRange = require('./split-text-at-range');

var _splitTextAtRange2 = _interopRequireDefault(_splitTextAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Split the inline nodes at a `range`, to optional `depth`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Number} depth (optiona)
 * @return {State} state
 */

function splitInlineAtRange(state, range) {
  var depth = arguments.length <= 2 || arguments[2] === undefined ? Infinity : arguments[2];
  var _state = state;
  var document = _state.document;

  // If the range is expanded, remove it first.

  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  // First split the text nodes.
  state = (0, _splitTextAtRange2.default)(state, range);
  document = state.document;

  // Find the children that were split.
  var _range = range;
  var startKey = _range.startKey;

  var firstChild = document.getDescendant(startKey);
  var secondChild = document.getNextText(firstChild);
  var parent = document.getClosestInline(firstChild);
  var d = 0;

  // While the parent is an inline parent, split the inline nodes.
  while (parent && d < depth) {
    firstChild = parent.merge({ nodes: _inline2.default.createList([firstChild]) });
    secondChild = _inline2.default.create({
      nodes: [secondChild],
      type: parent.type,
      data: parent.data
    });

    // Split the children.
    var grandparent = document.getParent(parent);
    var nodes = grandparent.nodes.takeUntil(function (n) {
      return n.key == firstChild.key;
    }).push(firstChild).push(secondChild).concat(grandparent.nodes.skipUntil(function (n) {
      return n.key == firstChild.key;
    }).rest());

    // Update the grandparent.
    document = grandparent == document ? document.merge({ nodes: nodes }) : document.updateDescendant(grandparent.merge({ nodes: nodes }));

    d++;
    parent = document.getClosestInline(firstChild);
  }

  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = splitInlineAtRange;