'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _text = require('../../models/text');

var _text2 = _interopRequireDefault(_text);

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Split the text nodes at a `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @return {State} state
 */

function splitTextAtRange(state, range) {
  var _state = state;
  var document = _state.document;

  // If the range is expanded, remove it first.

  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  // Split the text node's characters.
  var _range = range;
  var startKey = _range.startKey;
  var startOffset = _range.startOffset;

  var text = document.getDescendant(startKey);
  var characters = text.characters;

  var firstChars = characters.take(startOffset);
  var secondChars = characters.skip(startOffset);
  var firstChild = text.merge({ characters: firstChars });
  var secondChild = _text2.default.create({ characters: secondChars });

  // Split the text nodes.
  var parent = document.getParent(text);
  var nodes = parent.nodes.takeUntil(function (c) {
    return c.key == firstChild.key;
  }).push(firstChild).push(secondChild).concat(parent.nodes.skipUntil(function (n) {
    return n.key == firstChild.key;
  }).rest());

  // Update the nodes.
  parent = parent.merge({ nodes: nodes });
  document = document.updateDescendant(parent);
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = splitTextAtRange;