'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _addMarkAtRange = require('./add-mark-at-range');

var _addMarkAtRange2 = _interopRequireDefault(_addMarkAtRange);

var _normalizeMark = require('../../utils/normalize-mark');

var _normalizeMark2 = _interopRequireDefault(_normalizeMark);

var _removeMarkAtRange = require('./remove-mark-at-range');

var _removeMarkAtRange2 = _interopRequireDefault(_removeMarkAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Add or remove a `mark` from the characters at `range`, depending on whether
 * it's already there.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Mark or String} mark (optional)
 * @return {State} state
 */

function toggleMarkAtRange(state, range, mark) {
  mark = (0, _normalizeMark2.default)(mark);
  var document = state.document;

  // When the range is collapsed, do nothing.

  if (range.isCollapsed) return state;

  // Check if the mark exists in the range already.
  var marks = document.getMarksAtRange(range);
  var exists = marks.some(function (m) {
    return m.equals(mark);
  });

  return exists ? (0, _removeMarkAtRange2.default)(state, range, mark) : (0, _addMarkAtRange2.default)(state, range, mark);
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = toggleMarkAtRange;