'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _block = require('../../models/block');

var _block2 = _interopRequireDefault(_block);

var _normalizeNodeOrMarkProperties = require('../../utils/normalize-node-or-mark-properties');

var _normalizeNodeOrMarkProperties2 = _interopRequireDefault(_normalizeNodeOrMarkProperties);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wrap all of the blocks in a `range` in a new block with `properties`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {String or Object} properties
 * @return {State} state
 */

function wrapBlockAtRange(state, range, properties) {
  properties = (0, _normalizeNodeOrMarkProperties2.default)(properties);
  var _state = state;
  var document = _state.document;

  // Get the block nodes, sorted by depth.

  var blocks = document.getBlocksAtRange(range);
  var sorted = blocks.sort(function (a, b) {
    var da = document.getDepth(a);
    var db = document.getDepth(b);
    if (da == db) return 0;else if (da > db) return -1;else return 1;
  });

  // Get the lowest common siblings, relative to the highest block.
  var highest = sorted.first();
  var depth = document.getDepth(highest);
  var siblings = blocks.reduce(function (memo, block) {
    var sibling = document.getDepth(block) == depth ? block : document.getClosest(block, function (p) {
      return document.getDepth(p) == depth;
    });
    memo = memo.push(sibling);
    return memo;
  }, _block2.default.createList());

  // Wrap the siblings in a new block.
  var wrapper = _block2.default.create({
    nodes: siblings,
    type: properties.type,
    data: properties.data
  });

  // Replace the siblings with the wrapper.
  var first = siblings.first();
  var last = siblings.last();
  var parent = document.getParent(highest);
  var nodes = parent.nodes.takeUntil(function (n) {
    return n == first;
  }).push(wrapper).concat(parent.nodes.skipUntil(function (n) {
    return n == last;
  }).rest());

  // Update the parent.
  document = parent == document ? document.merge({ nodes: nodes }) : document.updateDescendant(parent.merge({ nodes: nodes }));

  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = wrapBlockAtRange;