'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _insertTextAtRange = require('./insert-text-at-range');

var _insertTextAtRange2 = _interopRequireDefault(_insertTextAtRange);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wrap the text in a `range` in a prefix/suffix.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {String} prefix
 * @param {String} suffix
 * @return {State} state
 */

function wrapTextAtRange(state, range, prefix) {
  var suffix = arguments.length <= 3 || arguments[3] === undefined ? prefix : arguments[3];

  // Insert text at the starting edge.
  var startKey = range.startKey;
  var endKey = range.endKey;

  var start = range.collapseToStart();
  state = (0, _insertTextAtRange2.default)(state, start, prefix);

  // Determine the new ending edge, and insert text there.
  var end = range.collapseToEnd();
  if (startKey == endKey) end = end.moveForward(prefix.length);
  state = (0, _insertTextAtRange2.default)(state, end, suffix);
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = wrapTextAtRange;