'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.operate = operate;

var _mark3 = require('../models/mark');

var _mark4 = _interopRequireDefault(_mark3);

var _character = require('../models/character');

var _character2 = _interopRequireDefault(_character);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Perform an operation with `properties`.
 *
 * @param {Transform} transform
 * @param {Object} operation
 * @return {Transform}
 */

function operate(transform, operation) {
  var type = operation.type;
  var state = transform.state;
  var _state = state;
  var document = _state.document;
  var selection = _state.selection;


  switch (type) {

    case 'insert-text':
      {
        var path = operation.path;
        var index = operation.index;
        var text = operation.text;
        var marks = operation.marks;

        var descendant = document.assertPath(path);
        descendant = descendant.insertText(index, text, marks);
        // TODO: fix marks
        // operation.marks = marks
        document = document.updateDescendant(descendant);
        break;
      }

    case 'remove-text':
      {
        var _path = operation.path;
        var _index = operation.index;
        var length = operation.length;

        var _descendant = document.assertPath(_path);
        _descendant = _descendant.removeText(_index, length);
        document = document.updateDescendant(_descendant);
        break;
      }

    case 'add-mark':
      {
        var _path2 = operation.path;
        var _index2 = operation.index;
        var _length = operation.length;
        var mark = operation.mark;

        var _descendant2 = document.assertPath(_path2);
        _descendant2 = _descendant2.addMark(_index2, _length, mark);
        document = document.updateDescendant(_descendant2);
        break;
      }

    case 'remove-mark':
      {
        var _path3 = operation.path;
        var _index3 = operation.index;
        var _length2 = operation.length;
        var _mark = operation.mark;

        var _descendant3 = document.assertPath(_path3);
        _descendant3 = _descendant3.removeMark(_index3, _length2, _mark);
        document = document.updateDescendant(_descendant3);
        break;
      }

    case 'update-mark':
      {
        var _path4 = operation.path;
        var _index4 = operation.index;
        var _length3 = operation.length;
        var _mark2 = operation.mark;
        var properties = operation.properties;

        var _descendant4 = document.assertPath(_path4);
        _descendant4 = _descendant4.updateMark(_index4, _length3, _mark2, properties);
        document = document.updateDescendant(_descendant4);
        break;
      }

    case 'insert-node':
      {
        var _path5 = operation.path;
        var _index5 = operation.index;
        var node = operation.node;

        var _descendant5 = document.assertPath(_path5);
        _descendant5 = _descendant5.insertNode(_index5, node);
        document = document.updateDescendant(_descendant5);
        break;
      }

    case 'remove-node':
      {
        var _path6 = operation.path;
        var _node = operation.node;

        var target = document.assertPath(_path6);
        var _descendant6 = document.getParent(target);
        var _index6 = _descendant6.nodex.indexOf(target);
        _descendant6 = _descendant6.removeNode(_index6);
        document = document.updateDescendant(_descendant6);
        break;
      }

    case 'update-node':
      {
        var _path7 = operation.path;
        var _properties = operation.properties;

        var _descendant7 = document.assertPath(_path7);
        _descendant7 = _descendant7.merge(_properties);
        document = document.updateDescendant(_descendant7);
        break;
      }

    case 'move-node':
      {
        var _path8 = operation.path;
        var newPath = operation.newPath;
        var newIndex = operation.newIndex;

        var _node2 = document.assertPath(_path8);
        var parent = document.getParent(_node2);
        var _target = document.assertPath(newPath);
        var _index7 = parent.nodes.indexOf(_node2);
        parent = parent.removeNode(_index7);
        _target = _target.insertNode(newIndex, _node2);
        document = document.updateDescendant(parent);
        document = document.updateDescendant(_target);
        break;
      }

    case 'update-selection':
      {
        var _properties2 = operation.properties;

        selection = selection.merge(_properties2);
        break;
      }

    default:
      {
        throw new Error('Invalid transform operation: "' + type + '".');
      }

  }

  state = state.merge({ document: document, selection: selection });
  transform.state = state;
  transform.operations.push(operation);
  return transform;
}