'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _data = require('../models/data');

var _data2 = _interopRequireDefault(_data);

var _typeOf = require('type-of');

var _typeOf2 = _interopRequireDefault(_typeOf);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Normalize the `properties` of a node or mark, which can be either a type
 * string or a dictionary of properties. If it's a dictionary, `data` is
 * optional and shouldn't be set if null or undefined.
 *
 * @param {String or Object} properties
 * @return {Object}
 */

function normalizeNodeOrMarkProperties() {
  var properties = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var ret = {};
  var type = (0, _typeOf2.default)(properties);

  switch (type) {
    case 'string':
      {
        ret.type = properties;
        break;
      }
    case 'object':
      {
        for (var key in properties) {
          if (key == 'data') {
            if (properties[key] != null) ret[key] = _data2.default.create(properties[key]);
          } else {
            ret[key] = properties[key];
          }
        }
        break;
      }
    default:
      {
        throw new Error('A `properties` argument must be an object or a string, but you passed: "' + type + '".');
      }
  }

  return ret;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = normalizeNodeOrMarkProperties;